---------------------------------------------------------------
--                                                           
--  PENCIL AND PAPER INTERPRETER
--  By: Martin Carlisle and Al White     
--                                                           
--  SIMULATION_GUI
--  Description : manually generated GUI objects in main window 
--                       
--  By: Martin Carlisle
--                                         
-- PENCIL AND PAPER INTERPRETER is free software; you can redistribute 
-- it and/or modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--  
-- PENCIL AND PAPER INTERPRETER is distributed in the hope that it will 
-- be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
with Mcc.Gui.Fonts;
with Main_Window;
with Controls;
with Mcc.Gui.Image;
with Mcc.Gui.Container.Window;
with Mcc.Gui.Widget.Text_Entry;
with Get_Instruction_Window;
with Ada.Integer_Text_IO;
with State;
with Instruction_Dialog;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Mcc.Gui.Colors;
with Pencil_And_Paper_Computer;
with PC_Dialog;
with Mcc.Gui.Widget.Scale;
package body Simulation_Gui is
   
   Address_Label : Mcc.Gui.Widget.Label.Label;
   Command_Label : Mcc.Gui.Widget.Label.Label;
   Value_Label   : Mcc.Gui.Widget.Label.Label;
   Decimal_Label : Mcc.Gui.Widget.Label.Label;
   PC_Arrow_Label: Mcc.Gui.Widget.Label.Label;
   
   Courier_10 : Mcc.Gui.Fonts.Font;
   Line_Height : constant := 15;
   Horizontal_Spacing : constant := 5;
   PC_Arrow_Left : constant := 2;
   PC_Arrow_Width : constant := 30;
   Location_Left : constant := PC_Arrow_Left + PC_Arrow_Width;
   Location_Width : constant := 75;
   Memory_Left : constant := Location_Left+Location_Width+Horizontal_Spacing;
   Memory_Width : constant := 95;
   Instruction_Left : constant := Memory_Left+Memory_Width+Horizontal_Spacing;
   Instruction_Width : constant := 120;
   Integer_Left : constant := Instruction_Left+Instruction_Width+Horizontal_Spacing;
   Integer_Width : constant := 60;
   Top_Row : constant := 18;
   
   Pressed_Y : Integer := -1;
   procedure Instruction_Click(
      Obj   : in out Mcc.Gui.Sized_Object'Class;
      Event : in     Mcc.Gui.Mouse_Event) is
      use Mcc.Gui;
      Window_X : Integer;
      Window_Y : Integer;
   begin
      Window_X := 250;
      Window_Y := 250;
      -- am getting clicks on double-clicking in File/Open or Save
      -- dialogs, also only want 1 dialog up at a time
      if State.Dialog_Displayed then
         return;
      end if;
      if Event.Button=Left and
         Event.Action=Release and
         Indexed_Label(Obj).Y = Pressed_Y then
         Controls.Stop;
         if not Mcc.Gui.Container.Window.Is_Open(
            Get_Instruction_Window.Get_Instruction_Window) then
            Instruction_Dialog.Instruction_Y :=
               Indexed_Label(Obj).Y;
            Get_Instruction_Window.Generate_and_Fill_Window;
            Mcc.Gui.Container.Window.Move(
               Obj => Get_Instruction_Window.Get_Instruction_Window,
               X   => Window_X,
               Y   => Window_Y);
            Mcc.Gui.Widget.Text_Entry.Highlight(
               Get_Instruction_Window.Instruction_Text);
            Mcc.Gui.Set_Key_Listener(
               Obj      => Get_Instruction_Window.Instruction_Text'access,
               Listener => Instruction_Dialog.Key_Callback'access);
         end if;
      elsif Event.Button=Left and
         Event.Action=Press then
         Pressed_Y := Indexed_Label(Obj).Y;
      end if;

   end Instruction_Click;

   procedure Location_Click(
      Obj   : in out Mcc.Gui.Sized_Object'Class;
      Event : in     Mcc.Gui.Mouse_Event) is
      use Mcc.Gui;
   begin
      -- am getting clicks on double-clicking in File/Open or Save
      -- dialogs, also only want 1 dialog up at a time
      if State.Dialog_Displayed then
         return;
      end if;
      if Event.Button=Left and
         Event.Action=Release then
         Controls.Stop;
         Pencil_And_Paper_Computer.Set_Program_Counter(
            Address  => Indexed_Label(Obj).Y);
         Draw_PC;
      end if;

   end Location_Click;


   function Five_Bit_Binary(X : in Memory_Location) return String is
      Result : String(1..5);
      Copy_X : Integer := X;
   begin
      if Copy_X >= 16 then
         Result(1) := '1';
         Copy_X := Copy_X - 16;
      else
         Result(1) := '0';
      end if;
      if Copy_X >= 8 then
         Result(2) := '1';
         Copy_X := Copy_X - 8;
      else
         Result(2) := '0';
      end if;
      if Copy_X >= 4 then
         Result(3) := '1';
         Copy_X := Copy_X - 4;
      else
         Result(3) := '0';
      end if;
      if Copy_X >= 2 then
         Result(4) := '1';
         Copy_X := Copy_X - 2;
      else
         Result(4) := '0';
      end if;
      if Copy_X = 1 then
         Result(5) := '1';
      else
         Result(5) := '0';
      end if;
      return Result;
   end Five_Bit_Binary;

   function Eight_Bit_Binary(X : in Memory_Value) return String is
      Result : String(1..8);
      Copy_X : Integer := X;
   begin
      if Copy_X >= 128 then
         Result(1) := '1';
         Copy_X := Copy_X - 128;
      else
         Result(1) := '0';
      end if;
      if Copy_X >= 64 then
         Result(2) := '1';
         Copy_X := Copy_X - 64;
      else
         Result(2) := '0';
      end if;
      if Copy_X >= 32 then
         Result(3) := '1';
         Copy_X := Copy_X - 32;
      else
         Result(3) := '0';
      end if;
      Result(4..8) := Five_Bit_Binary(Copy_X);
      return Result;
   end Eight_Bit_Binary;

   function Fourteen_Bit_Binary(X : in Accumulator_Value) return String is
      Result : String(1..14);
      Copy_X : Integer := X;
   begin
      if Copy_X >= 2**13 then
         Result(1) := '1';
         Copy_X := Copy_X - 2**13;
      else
         Result(1) := '0';
      end if;
      if Copy_X >= 2**12 then
         Result(2) := '1';
         Copy_X := Copy_X - 2**12;
      else
         Result(2) := '0';
      end if;
      if Copy_X >= 2**11 then
         Result(3) := '1';
         Copy_X := Copy_X - 2**11;
      else
         Result(3) := '0';
      end if;
      if Copy_X >= 2**10 then
         Result(4) := '1';
         Copy_X := Copy_X - 2**10;
      else
         Result(4) := '0';
      end if;
      if Copy_X >= 2**9 then
         Result(5) := '1';
         Copy_X := Copy_X - 2**9;
      else
         Result(5) := '0';
      end if;
      if Copy_X >= 2**8 then
         Result(6) := '1';
         Copy_X := Copy_X - 2**8;
      else
         Result(6) := '0';
      end if;
      Result(7..14) := Eight_Bit_Binary(Copy_X);
      return Result;
   end Fourteen_Bit_Binary;
   
   
   function Integer_Image3(x : in integer) return String is
      result : string(1..3);
   begin
      Ada.Integer_Text_IO.Put(To => Result, Item => x);
      return Result;
   end Integer_Image3;
   
   procedure Draw_PC is
      PC : Natural;
      Accumulator : Natural;
      IR : Natural;
   begin
      PC := Pencil_And_Paper_Computer.Get_Program_Counter;
      Accumulator := Pencil_And_Paper_Computer.Get_Accumulator;
      Mcc.Gui.Widget.Label.Move(
         Obj => PC_Arrow_Label,
         X   => PC_Arrow_Left,
         Y   => Top_Row+Line_Height*PC);
      Mcc.Gui.Widget.Label.Set_Text(
         Obj  => Main_Window.PC_Value,
         Text => Five_Bit_Binary(PC));
      Mcc.Gui.Widget.Label.Set_Text(
         Obj  => Main_Window.Accumulator_Value,
         Text => Fourteen_Bit_Binary(Accumulator));
      IR := Pencil_And_Paper_Computer.Get_Location(PC);
      Mcc.Gui.Widget.Label.Set_Text(
         Obj  => Main_Window.Instruction_Register,
         Text => Eight_Bit_Binary(IR));
   end Draw_PC;

   procedure Display_Instructions is
   begin
      for i in Memory_Labels'range loop
         Set_Text(
            Obj  => Memory_Labels(I),
            Text => Eight_Bit_Binary(
               Pencil_And_Paper_Computer.Get_Location(I)));
         Mcc.Gui.Widget.Label.Set_Text(
            Obj  => Instruction_Labels(I),
            Text => Pencil_And_Paper_Computer.Get_Location(I));
         Mcc.Gui.Widget.Label.Set_Text(
            Obj  => Integer_Labels(I),
            Text => Integer_Image3(
               Pencil_And_Paper_Computer.Get_Location(I)));
      end loop;
   end Display_Instructions;
   ----------------
   -- Initialize --
   ----------------

   procedure Initialize is
   begin
   
      Mcc.Gui.Fonts.Create(
         Obj    => Courier_10,
         Family => Mcc.Gui.Fonts.Monospaced,
         Size   => 10,
         Style  => Mcc.Gui.Fonts.Plain);

      Mcc.Gui.Widget.Label.Create(
         Obj     => PC_Arrow_Label,
         Parent  => Main_Window.Main_Window,
         X       => PC_Arrow_Left,
         Y       => Top_Row,
         Width   => PC_Arrow_Width,
         Height  => Line_Height,
         Justify => Mcc.Gui.Widget.Label.Left,
         Text    => "=>");
      Mcc.Gui.Fonts.Set_Font(
         Obj      => PC_Arrow_Label,
         New_Font => Courier_10);
      Mcc.Gui.Widget.Label.Set_Foreground_Color(
         Obj       => PC_Arrow_Label,
         New_Color => Mcc.Gui.Colors.Named_Color(Mcc.Gui.Colors.Red));
      Mcc.Gui.Widget.Label.Create(
         Obj     => Address_Label,
         Parent  => Main_Window.Main_Window,
         X       => Location_Left,
         Y       => Top_Row-Line_Height,
         Width   => Location_Width,
         Height  => Line_Height,
         Justify => Mcc.Gui.Widget.Label.Left,
         Text    => "Address");
      Mcc.Gui.Widget.Label.Create(
         Obj     => Address_Label,
         Parent  => Main_Window.Main_Window,
         X       => Memory_Left,
         Y       => Top_Row-Line_Height,
         Width   => Memory_Width,
         Height  => Line_Height,
         Justify => Mcc.Gui.Widget.Label.Left,
         Text    => "Value");
      Mcc.Gui.Widget.Label.Create(
         Obj     => Address_Label,
         Parent  => Main_Window.Main_Window,
         X       => Instruction_Left,
         Y       => Top_Row-Line_Height,
         Width   => Instruction_Width,
         Height  => Line_Height,
         Justify => Mcc.Gui.Widget.Label.Left,
         Text    => "Command");
      Mcc.Gui.Widget.Label.Create(
         Obj     => Address_Label,
         Parent  => Main_Window.Main_Window,
         X       => Integer_Left,
         Y       => Top_Row-Line_Height,
         Width   => 70,
         Height  => Line_Height,
         Justify => Mcc.Gui.Widget.Label.Left,
         Text    => "Decimal");
      for i in Memory_Labels'range loop
         Create(
            Obj     => Location_Labels(i),
            Parent  => Main_Window.Main_Window,
            X       => Location_Left,
            Y       => Top_Row+I*Line_Height,
            Width   => Location_Width,
            Height  => Line_Height,
            Justify => Mcc.Gui.Widget.Label.Left,
            Text    => Five_Bit_Binary(I));
         Location_Labels(I).Y := I;
         Mcc.Gui.Set_Mouse_Listener(
            Obj       => Location_Labels(i)'access,
            Listener  => Location_Click'access);
         Mcc.Gui.Fonts.Set_Font(
            Obj      => Location_Labels(i),
            New_Font => Courier_10);
         Create(
            Obj     => Memory_Labels(i),
            Parent  => Main_Window.Main_Window,
            X       => Memory_Left,
            Y       => Top_Row+I*Line_Height,
            Width   => Memory_Width,
            Height  => Line_Height,
            Justify => Mcc.Gui.Widget.Label.Left,
            Text    => "00000000");
         Memory_Labels(I).Y := I;
         Mcc.Gui.Set_Mouse_Listener(
            Obj       => Memory_Labels(i)'access,
            Listener  => Instruction_Click'access);
         Mcc.Gui.Fonts.Set_Font(
            Obj      => Memory_Labels(i),
            New_Font => Courier_10);
         Mcc.Gui.Widget.Label.Create(
            Obj     => Instruction_Labels(i),
            Parent  => Main_Window.Main_Window,
            X       => Instruction_Left,
            Y       => Top_Row+I*Line_Height,
            Width   => Instruction_Width,
            Height  => Line_Height,
            Justify => Mcc.Gui.Widget.Label.Left,
            Text    => "ADD");
         Mcc.Gui.Fonts.Set_Font(
            Obj      => Instruction_Labels(i),
            New_Font => Courier_10);
         Mcc.Gui.Widget.Label.Create(
            Obj     => Integer_Labels(i),
            Parent  => Main_Window.Main_Window,
            X       => Integer_Left,
            Y       => Top_Row+I*Line_Height,
            Width   => Integer_Width,
            Height  => Line_Height,
            Justify => Mcc.Gui.Widget.Label.Right,
            Text    => "0");
         Mcc.Gui.Fonts.Set_Font(
            Obj      => Integer_Labels(i),
            New_Font => Courier_10);
         Mcc.Gui.Set_Mouse_Listener(
            Obj       => Main_Window.PC_Value'access,
            Listener  => PC_Dialog.PC_Click'access);
         Mcc.Gui.Set_Mouse_Listener(
            Obj       => Main_Window.Accumulator_Value'access,
            Listener  => PC_Dialog.Accumulator_Click'access);
      end loop;
      Mcc.Gui.Widget.Scale.Set(
         Obj      => Main_Window.Speed_Scale,
         Location => 50);
      Pencil_And_Paper_Computer.Reset_Program_Counter;
      Display_Instructions;
   end Initialize;

   ---------------------
   -- Set_Instruction --
   ---------------------

   procedure Set_Instruction (Which : in Memory_Location) is
   begin
      Set_Text(
         Obj  => Memory_Labels(Which),
         Text => Eight_Bit_Binary(
            Pencil_And_Paper_Computer.Get_Location(Which)));
      Mcc.Gui.Widget.Label.Set_Text(
         Obj  => Instruction_Labels(Which),
         Text => Pencil_And_Paper_Computer.Get_Location(Which));
      Mcc.Gui.Widget.Label.Set_Text(
         Obj  => Integer_Labels(Which),
         Text => Integer_Image3(
            Pencil_And_Paper_Computer.Get_Location(Which)));
   end Set_Instruction;

end Simulation_Gui;

