---------------------------------------------------------------
--                                                           
--  PENCIL AND PAPER INTERPRETER
--  By: Martin Carlisle and Al White     
--                                                           
--  SIMULATION_GUI
--  Description : manually generated GUI objects in main window 
--                       
--  By: Martin Carlisle
--                                         
-- PENCIL AND PAPER INTERPRETER is free software; you can redistribute 
-- it and/or modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--  
-- PENCIL AND PAPER INTERPRETER is distributed in the hope that it will 
-- be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
with Mcc.Gui.Widget.Label;
package Simulation_Gui is
 
   subtype Memory_Location is Natural range 0..31;
   subtype Memory_Value is Natural range 0..255;
   subtype Accumulator_Value is Natural range 0..2**14-1;
   
   
   type Indexed_Label is new Mcc.Gui.Widget.Label.Label with record
      Y : Natural;
   end record;
   
   Location_Labels   : array(Memory_Location) of aliased Indexed_Label;
   Memory_Labels     : array(Memory_Location) of aliased Indexed_Label;
   Instruction_Labels: array(Memory_Location) of Mcc.Gui.Widget.Label.Label;
   Integer_Labels    : array(Memory_Location) of Mcc.Gui.Widget.Label.Label;

      
   -------------------------------------------------------
   -- Set_Instruction
   --
   -- moves the arrow to the memory location specified
   -------------------------------------------------------
   procedure Set_Instruction(Which : in Memory_Location);
   

   -------------------------------------------------------
   -- Initialize
   --
   -- draw all of the labels with memory all set to 0
   -- and a room of size Max_Room_Size and the robot in the 
   -- lower left corner.  Only call this once, or
   -- Already_Initialized will be raised.
   -------------------------------------------------------
   Already_Initialized : exception;
   procedure Initialize;
   
   
   --------------------------------------------------------
   -- call when a new program has been loaded to display it
   --------------------------------------------------------
   procedure Display_Instructions;
   
   --------------------------------------------------------
   -- call when the program counter has changed
   --------------------------------------------------------
   procedure Draw_PC;
   
   --------------------------------------------------------
   -- get binary strings
   --------------------------------------------------------
   function Five_Bit_Binary(X : in Memory_Location) return String;
   function Eight_Bit_Binary(X : in Memory_Value) return String;
   function Fourteen_Bit_Binary(X : in Accumulator_Value) return String;
end Simulation_Gui;